from qtrf.widgets.file import FileWidget

class UserCorrectionsWidget(FileWidget):
    def __init__(self, parent=None):
        FileWidget.__init__(self, parent)
        self.filter = 'User Corrections File (*.uco *.ucor)'



from .helpers                           import prefix_map, units
from qtrf.widgets.float_value.line_edit import FloatValueLineEdit


class MetersLineEdit(FloatValueLineEdit):
    def __init__(self, parent=None):
        FloatValueLineEdit.__init__(self, parent)
        self.prefix_keys       = prefix_map
        self.include_negative = False
        self.units            = units
        self.update_validator()



from .float_value        import FloatValueMixin
from .helpers            import float_validator
from .prefix_keys        import PrefixKeysMixin
from .special_keys       import SpecialKeysMixin
from PySide2.QtCore      import Slot
from qtrf.numeric_suffix import to_decimal


class FloatLineEditMixin(SpecialKeysMixin, PrefixKeysMixin, FloatValueMixin):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.include_negative = True
        self.update_validator()
        self.connect_qt_signals_and_slots()

    @Slot()
    def focusInEvent(self, event):
        super().focusInEvent(event)
        self.selectAll()

    @Slot()
    def focusOutEvent(self, event):
        super().focusOutEvent(event)
        self.enter_value()

    @Slot()
    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        self.selectAll()

    def connect_qt_signals_and_slots(self):
        self.special_key_pressed.connect(self.enter_value)
        self.prefix_key_pressed.connect(self.enter_value_with_prefix)


    # helpers

    def enter_value(self):
        self.value = to_decimal(self.text(), self.units, self.prefix_keys)
        self.selectAll()

    def enter_value_with_prefix(self, prefix):
        # base
        base_value = to_decimal(self.text(), self.units)
        if base_value is None:
            # clear value
            self.value = None
            return

        # get prefix value
        prefix = chr(prefix)
        if not prefix in self.prefix_keys:
            prefix = prefix.lower()
        prefix_value = self.prefix_keys[prefix]

        # set
        self.value = base_value * prefix_value
        self.selectAll()

    def update_validator(self):
        _validator = float_validator(self.prefix_keys, self.units, self.include_negative)
        self.setValidator(_validator)



from PySide2.QtCore    import Signal, Slot, Qt


SPECIAL_KEYS = [Qt.Key_Space, Qt.Key_Enter, Qt.Key_Return]


class SpecialKeysMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.special_keys = SPECIAL_KEYS

    special_key_pressed = Signal(int)

    # helpers

    @Slot()
    def keyPressEvent(self, event):
        key = event.key()
        if key in self.special_keys:
            event.accept()
            self.special_key_pressed.emit(key)
        else:
            super().keyPressEvent(event)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'file.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_FileWidget:
    def setupUi(self, FileWidget):
        if not FileWidget.objectName():
            FileWidget.setObjectName(u"FileWidget")
        self.horizontalLayout = QHBoxLayout(FileWidget)
#ifndef Q_OS_MAC
        self.horizontalLayout.setSpacing(-1)
#endif
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.filename = QLineEdit(FileWidget)
        self.filename.setObjectName(u"filename")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filename.sizePolicy().hasHeightForWidth())
        self.filename.setSizePolicy(sizePolicy)
        self.filename.setFocusPolicy(Qt.NoFocus)
        self.filename.setReadOnly(True)

        self.horizontalLayout.addWidget(self.filename)

        self.open = QPushButton(FileWidget)
        self.open.setObjectName(u"open")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.open.sizePolicy().hasHeightForWidth())
        self.open.setSizePolicy(sizePolicy1)
        self.open.setMinimumSize(QSize(60, 0))
        self.open.setMaximumSize(QSize(60, 16777215))

        self.horizontalLayout.addWidget(self.open)

        self.clear = QPushButton(FileWidget)
        self.clear.setObjectName(u"clear")
        sizePolicy1.setHeightForWidth(self.clear.sizePolicy().hasHeightForWidth())
        self.clear.setSizePolicy(sizePolicy1)
        self.clear.setMinimumSize(QSize(60, 0))
        self.clear.setMaximumSize(QSize(60, 16777215))

        self.horizontalLayout.addWidget(self.clear)

        self.horizontalLayout.setStretch(0, 1)

        self.retranslateUi(FileWidget)

        QMetaObject.connectSlotsByName(FileWidget)
    # setupUi

    def retranslateUi(self, FileWidget):
        self.open.setText(QCoreApplication.translate("FileWidget", u"...", None))
        self.clear.setText(QCoreApplication.translate("FileWidget", u"X", None))
        pass
    # retranslateUi




from .helpers                         import points_validator
from qtrf.widgets.int_value.line_edit import IntValueLineEdit


class PointsLineEdit(IntValueLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.update_validator()

    def update_validator(self):
        self.setValidator(points_validator())



from .ui_file                   import Ui_FileWidget
from pathlib                    import Path
from PySide2.QtCore             import Signal, Slot
from PySide2.QtWidgets          import QFileDialog, QWidget


class FileWidget(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.ui = Ui_FileWidget()
        self.ui.setupUi(self)
        self.caption    = 'Open...'
        self.filter     = ''
        self._filename  = None
        self._last_path = Path('.')
        self.ui.open .clicked.connect(self.choose_file)
        self.ui.clear.clicked.connect(self.clear)

    @property
    def filename(self):
        if self._filename:
            return str(self._filename)
        else:
            return None

    @filename.setter
    def filename(self, filename):
        if not filename and self.filename:
            # clear
            self._filename = None
            self._update_filename()
            self.filename_changed.emit(None)
            return

        if filename and filename != self.filename:
            self._filename  = Path(filename)
            self._last_path = self._filename.parent
            self._update_filename()
            self.filename_changed.emit(self.filename)

    filename_changed = Signal(str)

    @Slot()
    def choose_file(self, dir=None):
        filename = self._dialog(dir)
        if not filename:
            return
        self.filename = filename

    @Slot()
    def clear(self):
        self.filename = None

    # helpers

    def _dialog(self, dir=None):
        return QFileDialog.getOpenFileName(self.window(),
                                           self.caption,
                                           dir or str(self._last_path),
                                           self.filter)[0]
    def _update_filename(self):
        if self._filename:
            self.ui.filename.setText(Path(self._filename).name)
        else:
            self.ui.filename.clear()



def type_or_none(Type, value):
    if value is None or value == '':
        return None
    # else
    return Type(value)


def float_or_none(value):
    return type_or_none(float, value)


def int_or_none(value):
    return type_or_none(int, value)



from PySide2.QtWidgets           import QLineEdit
from qtrf.mixins.float_line_edit import FloatLineEditMixin


class FloatValueLineEdit(FloatLineEditMixin, QLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)



from .helpers import validator


class ValidatorMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.update_validator()


    # helpers

    def update_validator(self):
        self.setValidator(validator())



from qtrf.widgets.file import FileWidget

class TouchstoneWidget(FileWidget):
    def __init__(self, parent=None):
        FileWidget.__init__(self, parent)
        self.ports = '*'

    @property
    def ports(self):
        return self._ports
    @ports.setter
    def ports(self, value):
        self._ports  = value
        self.filter = 'Touchstone File (*.s{}p)'.format(self._ports)



from   .command_line     import parse_args
import code
from   PySide2.QtWidgets import QApplication
import qtrf.widgets
import sys


def is_qtrf_widget(name):
    return name in dir(qtrf.widgets)


def main():
    # parse CLI input
    args = parse_args()

    # create qt app
    q_app  = QApplication()

    # is qtrf.widget_class?
    try:
        WidgetClass = getattr(qtrf.widgets, args.widget_class)
    except AttributeError:
        print("error 404: {widget_class} not found in qtrf".format(widget_class=args.widget_class))
        sys.exit(404)

    # create
    widget = WidgetClass()
    widget.show()

    # run
    return_code = q_app.exec_()

    # interactive mode?
    if args.interactive:
        banner = '{widget_class} => widget'.format(widget_class=args.widget_class)
        code.interact(banner, local=locals())

    sys.exit(return_code)


if __name__ == '__main__':
    main()



from   .ports         import to_list, to_str
import builtins
from   PySide2.QtCore import Signal


class PortsListMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._list = []

    @property
    def list(self):
        return self._list

    @list.setter
    def list(self, list):
        if self.list == list:
            # list is unchanged
            return

        # set
        self._list = list
        self.update_text()

        # list has changed:
        # emit signal
        self.list_changed.emit(self.list)

    list_changed = Signal(builtins.list)

    # helpers

    def update_text(self):
        self.setText(to_str(self.list))



from PySide2.QtCore    import QRegExp
from PySide2.QtGui     import QRegExpValidator


# hostname regex
# https://stackoverflow.com/a/106223/1390788
hostname   = r'(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])'


# ip address regex
from_zero  = r'(((00)?\d)|([01]?\d\d)|(2[0-4]\d)|(25[0-5]))'
from_one   = r'(((0|00)?[1-9])|(0?[1-9]\d)|(1\d\d)|(2[0-4]\d)|(25[0-5]))'
dot        = r'\.'
ip_address = f'{from_one}{dot}{from_zero}{dot}{from_zero}{dot}{from_one}'


IP_ADDRESS_REGEX = f'(({ip_address})|({hostname}))'


def ip_address_validator():
    return QRegExpValidator(QRegExp(IP_ADDRESS_REGEX))



from .float_line_edit import FloatLineEditMixin
from .float_value     import FloatValueMixin
from .int_line_edit   import IntLineEditMixin
from .prefix_keys     import PrefixKeysMixin
from .special_keys    import SpecialKeysMixin



from qtrf.widgets.float_value.line_edit import FloatValueLineEdit


class CelsiusLineEdit(FloatValueLineEdit):
    def __init__(self, parent=None):
        FloatValueLineEdit.__init__(self, parent)
        self.prefix_keys    = []
        self.units          = '°C'
        self.decimal_places = 1
        self.update_validator()



from   ddt                             import ddt, data
import re
from   qtrf.widgets.ports.mixins.regex import ports_list_regex
from   qtrf.widgets.ports.mixins.ports import to_list, to_str
import unittest

@ddt
class TestUnits(unittest.TestCase):
    @data({'text': '',           'list': []},
          {'text': '1',          'list': [1]},
          {'text': '1,2',        'list': [1,2]},
          {'text': '1, 2, 3, 4', 'list': [1,2,3,4]},
          {'text': '1-4',        'list': [1,2,3,4]},
          {'text': '1, 3, 1-4',  'list': [1,2,3,4]})
    def test_to_list(self, data):
        text = data['text']
        list = data['list']
        self.assertEqual(to_list(text), list)

    @data({'list': [],                'text': ''    },
          {'list': [1],               'text': '1'   },
          {'list': [1,2],             'text': '1-2' },
          {'list': [1,3],             'text': '1, 3'},
          {'list': [1,4,9,10,11],     'text': '1, 4, 9-11'},
          {'list': [1,2,3,4,9,10,11], 'text': '1-4, 9-11' })
    def test_to_str(self, data):
        list = data['list']
        text = data['text']
        self.assertEqual(to_str(list), text)

    @data({'text': '',           'valid': False},
          {'text': '1',          'valid': True},
          {'text': '1,2',        'valid': True},
          {'text': '1, 2',       'valid': True},
          {'text': '1-4',        'valid': True},
          {'text': '1-2,3, 4-5', 'valid': True})
    def test_port_list_regex(self, data):
        text   = data['text']
        valid  = data['valid']
        _valid = bool(re.match(ports_list_regex, text))
        self.assertEqual(_valid, valid)



from qtrf.widgets.float_value.label import FloatValueLabel


class dBmLabel(FloatValueLabel):
    def __init__(self, parent=None):
        FloatValueLabel.__init__(self, parent)
        self.prefix_keys = []
        self.units       = 'dBm'



from decimal import Decimal, InvalidOperation, ROUND_HALF_UP


prefix_map = {
    'f': Decimal('1e-15'),
    'p': Decimal('1e-12'),
    'n': Decimal('1e-9'),
    'u': Decimal('1e-6'),
    'm': Decimal('1e-3'),
    '' : Decimal('1'),
    'K': Decimal('1e3'),
    'M': Decimal('1e6'),
    'G': Decimal('1e9'),
    'T': Decimal('1e12')
}



import argparse


description='dtool for live debug of qtrf widgets'


def parse_args():
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('-i', '--interactive', action='store_true', help='show python prompt before exit')
    parser.add_argument('widget_class')
    return parser.parse_args()



from PySide2.QtWidgets import QLineEdit
from qtrf.mixins       import IntLineEditMixin


class IntValueLineEdit(IntLineEditMixin, QLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
